
/* jquery ui css fix */
.ui-widget-overlay {
    opacity: 0.5;
}
.ui-widget-header {
    border: none;
    background-color: #79aec8;
    color: #fff;
}

.dialog-btn-primary {
    background-color: #417690;
    border: 1px solid #417690;
    color: #fff;
}
.dialog-btn-primary:hover,
.dialog-btn-primary:active,
.dialog-btn-primary:visited {
    background-color: #205067;
    border: 1px solid #205067;
    color: #fff;
}
