<li class="{{classes}}">
    <div href="javascript:;" class="PowerAdminChangelistToolbarDropDown">
        <span class="PowerAdminChangelistToolbarDropDownButton">{{label}}&nbsp;<i class="{{icon}}"></i></span>
        <div class="PowerAdminChangelistToolbarDropDownContent">
            {% for button in buttons %}
            <span class="PowerAdminChangelistToolbarDropDownItem">
                <nobr><a class="{{button.classes}}" href="{{button.href}}">{{button.label}}&nbsp;<i class="{{button.icon}}"></i></a></nobr>
            </span>
            {% endfor %}
        </div>
    </div>
</li>
