{% load i18n %}
<div class="django_power_admin_widgets_config_table_wrapper">
<textarea name="{{ widget.name }}"{% include "django/forms/widgets/attrs.html" %} style="display: none;">{% if widget.value %}{{ widget.value }}{% endif %}</textarea>
<table class="django_power_admin_widgets_config_table">
    <tr class="django_power_admin_widgets_config_table_title">
        <th class="django_power_admin_widgets_config_table_title_key">{% trans "Config Key" %}</th>
        <th class="django_power_admin_widgets_config_table_title_value">{% trans "Config Value" %}</th>
        <th class="django_power_admin_widgets_config_table_title_remove">{% trans "Remove Item" %}</th>
    </tr>
    <tr class="django_power_admin_widgets_config_table_item">
        <td class="django_power_admin_widgets_config_table_item_key"><input type="text" /></td>
        <td class="django_power_admin_widgets_config_table_item_value"><input type="text" /></td>
        <td class="django_power_admin_widgets_config_table_item_remove">
            <a href="#">
                <i class="fas fa-trash"></i>&nbsp;{% trans "Delete" %}
            </a>
        </td>
    </tr>
    <tr class="django_power_admin_widgets_config_table_add_another_config">
        <td colspan="3"><a href="#" class="django_power_admin_widgets_config_table_add_another_config_button"><i class="fas fa-plus"></i>&nbsp;{% trans "Add another config item" %}</a></td>
    </tr>
    <tr class="django_power_admin_widgets_config_table_item_template" style="display: none;">
        <td class="django_power_admin_widgets_config_table_item_key"><input type="text" /></td>
        <td class="django_power_admin_widgets_config_table_item_value"><input type="text" /></td>
        <td class="django_power_admin_widgets_config_table_item_remove">
            <a href="#">
                <i class="fas fa-trash"></i>&nbsp;{% trans "Delete" %}
            </a>
        </td>
    </tr>
</table>
</div>
