{% load i18n %}

<h3>{{ title }}</h3>
<ul class="DateRangeFilter">
    <li>
        <form method="GET" action="?">
            <input
                type="text"
                name="{{spec.input_start_name}}"
                value="{{spec.input_start_value}}"
                class="DateRangeFilterInput DateRangeFilterStartInput"
                autocomplete="off"
                placeholder="{{spec.input_start_placeholder}}"
                />
            -
            <input
                type="text"
                name="{{spec.input_end_name}}"
                value="{{spec.input_end_value}}"
                class="DateRangeFilterInput DateRangeFilterEndInput"
                autocomplete="off"
                placeholder="{{spec.input_end_placeholder}}"
                />

            <input type="button" value="{% trans "Search" %}"  class="DateRangeFilterSubmitButton" />
            <input type="button" value="{% trans "Reset" %}" class="DateRangeFilterResetButton" />
        </form>
    </li>
</ul>

</select>
