.django_power_admin_widgets_config_table_item_key input {
    width: 200px;
}
.django_power_admin_widgets_config_table_item_value input {
    width: 400px;
}
.django_power_admin_widgets_config_table_item_remove a {
    line-height: 24px;
}