{% load i18n %}

<h3>{{ title }}</h3>
<ul class="MultipleSelectFieldFilter">
    <li>
        <select class="django_horizontal_list_filter_field django_horizontal_list_filter_field-{{spec.field.name}}" multiple>
            {% for choice in choices %}
            <option value="{% django_horizontal_list_filter_clean_query_string cl spec choice.query_string %}"{% if choice.selected %} selected="selected"{% endif %}>{{ choice.display }}</option>
            {% endfor %}
        </select>
    </li>
</ul>

