

.DateRangeFilter .DateRangeFilterStartInput,
.DateRangeFilter .DateRangeFilterEndInput {
    display: inline-block;
    width: 80px;
}
.DateRangeFilter input[type=button] {
    padding: 4px 8px;
}
.DateRangeFilter input[type=button].DateRangeFilterResetButton {
    background-color: #999;
}

/* Work with django-horizontal-list-filter, no need inside search and reset button in the filter block */
.django-horizontal-list-filter-group .DateRangeFilterSubmitButton,
.django-horizontal-list-filter-group .DateRangeFilterResetButton {
    display: none;
}
