
.TextInputFieldFilter .TextInputFieldFilterInput {
    display: block;
    width: 90%;
}
.TextInputFieldFilter input[type=button] {
    padding: 4px 8px;
}
.TextInputFieldFilter input[type=button].TextInputFieldFilterResetButton {
    background-color: #999;
}

.django-horizontal-list-filter-group .TextInputFieldFilterSubmitButton,
.django-horizontal-list-filter-group .TextInputFieldFilterResetButton {
    display: none;
}
